#include "helperFann.h"
#include <stdio.h>

/* Simple Matlab-function for loading a FANN-trained neural net.
 * Built using FANN 2.1.0 beta.
 * For use with the bindings available from:
 * http://sumowiki.intec.ugent.be/index.php/FANN_Bindings 
 * 
 * Author: Martin Etnestad Johansen, 2009
 *
 * Note: mexErrMsgTxt caused Matlab to crash, but mexPrintf works fine.
 */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
    
    const mxArray *data;

    // Output error message if incorrect number of arguments were given.
    if (nrhs != 1 || nlhs != 1) {
        mexPrintf("loadFann usage: 'ann = loadFann(filename)'");
    }
    else {
        // Get a pointer to a C-style string containing the filename.
        data = prhs[0];
        char *filename = mxArrayToString(data);

        // Load the net, then free the string memory.
        struct fann *ann = fann_create_from_file(filename);
        mxFree(filename);

        // Output an error if net could not be loaded.
        if (ann == NULL) {
            mexPrintf("Invalid file or filename!");
        }
        else {
            // Get the number of layers.
            size_t i, numLayers = fann_get_num_layers(ann);

            // Allocate memory to hold the neuron count for the layers.
            double *neurons_d = mxCalloc(numLayers, sizeof(double));
            unsigned int *neurons_uint = mxCalloc(numLayers, sizeof(unsigned int));

            // Get the neuron count, then copy/cast into the double array.
            fann_get_layer_array(ann, neurons_uint);            
            for (i = 0; i < numLayers; i++) {
               neurons_d[i] = (double) neurons_uint[i]; 
            }

            // Make a double matrix, and set the double array as the data.
            mxArray *neuronVector = mxCreateDoubleMatrix(1, numLayers, mxREAL);        
            mxSetData(neuronVector, neurons_d);

            // Make the Matlab struct (this function is in helperFann.c) to return.
            plhs[0] = createMatlabStruct(ann, neuronVector, fann_get_connection_rate(ann));

            // Free used memory. neurons_d should NOT be freed, as it is now in use!
            fann_destroy(ann);
            mxFree(neurons_uint);
        }
    }
}
