#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>


#include "fannextras.h"


// Externs and globals.
extern struct main_data_struct data;

double *data_input, *data_output;



/* Training callback.
 *
 * This function outputs information after every epoch and does verification at the specified intervals,
 * saving the best neural net to the specified output file.
 * (Optionally, to stop the training, the function must return -1.)
 */
int FANN_API TrainingCallback(struct fann *ann,
                              struct fann_train_data *train,
                              unsigned int max_epochs,
                              unsigned int epochs_between_reports,
                              float desired_error,
                              unsigned int epochs)
{
    static int epochs_since_verif = 0;

    // Get current training error.
    data.error_train_curr = fann_get_MSE(data.ann);

    // Output information about the training.
    printf("\nEpoch #%u\n\tTraining error: %f\n\t        change: %f\n\t    per output: %f\n",
           epochs,
           NUM_OUTPUTS*data.error_train_curr,
           NUM_OUTPUTS*(data.error_train_curr - data.error_train_prev),
           data.error_train_curr);

    // Perform verification at specified intervals.
    epochs_since_verif++;
    if (epochs_since_verif == data.verif_epochs) {
        epochs_since_verif = 0;

        // Shuffle the verification data. (Might be good for big datasets..)
        fann_shuffle_train_data(data.verif_data);

        // Get current verification error.
        data.error_verif_curr = fann_test_data(data.ann, data.verif_data);

        // Output information about the verification.
        printf("\t  Verif. error: %f\n\t        change: %f\n\t    per output: %f\n",
               NUM_OUTPUTS*data.error_verif_curr,
               NUM_OUTPUTS*(data.error_verif_curr - data.error_verif_prev),
               data.error_verif_curr);

        // If current verification error is less than the previously
        // recorded best, output and update info, then save the net.
        if (data.error_verif_curr < data.error_verif_best) {
            printf("Better net found.\n\t        change: %f\n",
                   NUM_OUTPUTS*(data.error_verif_curr - data.error_verif_best));

            data.error_verif_best = data.error_verif_curr;
            data.best_epoch = epochs;
            fann_save(data.ann, data.net_out_file);
        }
        // Otherwise, just output info about the best verification epoch so far.
        else {
            printf("Best epoch: #%i\n\t    difference: %f\n",
                   data.best_epoch,
                   NUM_OUTPUTS*(data.error_verif_curr - data.error_verif_best));
        }
    }

    // For especially sequential backprop, the learning rate should be reduced for every epoch.
    if ((data.algorithm == FANN_TRAIN_INCREMENTAL) || (data.algorithm == FANN_TRAIN_BATCH)) {
        float temp_rate = fann_get_learning_rate(data.ann);
        fann_set_learning_rate(data.ann, temp_rate/data.param3);
    }

    // Update variables that hold previous errors.
    data.error_verif_prev = data.error_verif_curr;
    data.error_train_prev = data.error_train_curr;

    // Shuffle the training data, required for sequential training.
    // (And might help if the dataset is huge for other algorithms?)
    fann_shuffle_train_data(data.train_data);

    return 0;
}


/* Load binary training data
 *
 * Load training data stored as doubles in Matlab (misc/SaveBinaryData.m)
 * The first 12 bytes indicate the number of training vectors,
 * and dimensionality of input & output.
 */
struct fann_train_data *TrainingDataLoad(char *filename)
{
    FILE *filepointer = NULL;
    data_input = NULL;
    data_output = NULL;

    struct fann_train_data *data_struct = NULL;

    // Try to open file for reading.
    filepointer = fopen(filename, "r");

    // Proceed with reading the file.
    if (filepointer != NULL) {
        uint32_t num_data, num_input, num_output, read_values;

        //fscanf(filepointer, "%u %u %u", &num_data, &num_input, &num_output);
        read_values = fread(&num_data, sizeof(uint32_t), 1, filepointer);
        read_values += fread(&num_input, sizeof(uint32_t), 1, filepointer);
        read_values += fread(&num_output, sizeof(uint32_t), 1, filepointer);

        // Allocate space for the input and output data.
        if (read_values == 3) {
            data_input = (double*) calloc(num_data*num_input, sizeof(double));
            data_output = (double*) calloc(num_data*num_output, sizeof(double));
        }
        else {
            printf("Could not read data set specs!\n");
        }

        // Read the data. Input vectors are first, then output.
        // (Everything is doubles!)
        if (data_input != NULL && data_output != NULL) {
            printf("Reading %u vectors each of dimensionality %u and %u.\n", num_data, num_input, num_output);

            read_values = fread(data_input, num_input*sizeof(double), num_data, filepointer);
            read_values += fread(data_output, num_output*sizeof(double), num_data, filepointer);

            // Proceed to populate the FANN training set with these training vectors.
            if (read_values == 2*num_data) {
                data_struct = fann_create_train_from_callback(num_data, num_input, num_output, &TrainingDataMove);
            }
            else {
                printf("Could not read correct number of vectors!\n");
            }

            free(data_input);
            free(data_output);
        }
        else {
            printf("Could not allocate space for data!\n");
            free(data_input);
            free(data_output);
        }

        fclose(filepointer);
    }

    return (data_struct);
}


/* Populate FANN training data struct with the read data.
 *
 */
void FANN_API TrainingDataMove(unsigned int num_data, unsigned int num_input, unsigned int num_output, fann_type *fann_in, fann_type *fann_out)
{
    unsigned int i;

    for (i = 0; i < num_input; i++) {
        fann_in[i] = (fann_type) data_input[(num_data*num_input + i)];
    }

    for (i = 0; i < num_output; i++) {
        fann_out[i] = (fann_type)  data_output[(num_data*num_output + i)];
    }
}
