/* Command line tool to train FANN networks
 *
 * See line 55 - 66 for usage directions.
 *
 * The network structure is hard coded in the program.
 * Update the definitions at the top of fannextras.h
 * as well as layer_neurons on line 51 here to change.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


#include "fannextras.h"


// Define command line argument order and position.
#define ARG_TRAIN_FILE     1
#define ARG_VERIF_FILE     2
#define ARG_NET_OUT_FILE   3
#define ARG_ALGORITHM      4
#define ARG_NET_IN_FILE    5
#define ARG_MAX_EPOCHS     6
#define ARG_VERIF_EPOCHS   7
#define ARG_ERROR          8
#define ARG_PARAM1         9
#define ARG_PARAM2         10
#define ARG_PARAM3         11

#define NUM_ARGS           11



// Put all data in here, since TrainingCallback needs some of them.
// (This struct is declared in fannextras.h.)
struct main_data_struct data  = {NULL, NULL, NULL, NULL,
                                  -1, 0, 0, 0,
                                  0.0f, 0.0f, 0.0f, 0.0f,
                                  NULL, NULL, NULL,
                                  0.0f, 0.0f, 0.0f, 0.0f, 0.0f};


// Start main program.
int main(int argc, char *argv[])
{
    // Neuron count for the layers. (First one is automatically linear!)
    //    unsigned int layer_neurons[NUM_LAYERS] = {NUM_INPUTS, NUM_HIDDEN, NUM_HIDDEN, NUM_OUTPUTS};
    unsigned int layer_neurons[NUM_LAYERS] = {NUM_INPUTS, NUM_HIDDEN, NUM_OUTPUTS};


    // Only output usage text if too few arguments were supplied.
    if (argc < NUM_ARGS+1) {
        printf("Parameters:\n"
                "\t1. Training file\n"
                "\t2. Verification file\n"
                "\t3. Network output file\n"
                "\t4. Algorithm (S, B, R, Q -> seq., batch, rprop, quick)\n"
                "\t5. Network input file (if invalid, new net is created)\n"
                "\t6. Max epochs\n"
                "\t7. Epochs between verifications\n"
                "\t8. Desired error\n"
                "\t9. Parameter 1 (training/grow/decay)\n"
                "\t10. Parameter 2 (momentum/shrink/step change)\n"
                "\t11. Parameter 3 (divisor for learning rate per verification)\n"
                "\nTraining/momentum: S/B, Grow/shrink: R, Decay/step change: Q\n");
        exit(1);
    }

    // Get pointers to string arguments.
    data.train_file = argv[ARG_TRAIN_FILE];
    data.verif_file = argv[ARG_VERIF_FILE];
    data.net_out_file = argv[ARG_NET_OUT_FILE];
    data.net_in_file = argv[ARG_NET_IN_FILE];

    // To check if something went wrong during reading of numerical arguments.
    int read_args = 0;

    // Get numerical arguments.
    read_args += sscanf(argv[ARG_MAX_EPOCHS], "%i", &data.max_epochs);
    read_args += sscanf(argv[ARG_VERIF_EPOCHS], "%i", &data.verif_epochs);
    read_args += sscanf(argv[ARG_ERROR], "%f", &data.target_error);
    read_args += sscanf(argv[ARG_PARAM1], "%f", &data.param1);
    read_args += sscanf(argv[ARG_PARAM2], "%f", &data.param2);
    read_args += sscanf(argv[ARG_PARAM3], "%f", &data.param3);

    if (read_args != 6) {
        printf("Numerical argument error!\n");
        exit(1);
    }


    // Select training algorithm according to argument, and output info. Quit if invalid method.
    switch (tolower(*argv[ARG_ALGORITHM])) {
        case 's':
            data.algorithm = FANN_TRAIN_INCREMENTAL;
            printf("Doing sequential training with learning rate = %f and momentum = %f\n", data.param1, data.param2);
            break;

        case 'b':
            data.algorithm = FANN_TRAIN_BATCH;
            printf("Doing batch training with learning rate = %f and momentum = %f\n", data.param1, data.param2);
            break;

        case 'r':
            data.algorithm = FANN_TRAIN_RPROP;
            printf("Doing Rprop training with growth factor = %f and shrink factor = %f\n", data.param1, data.param2);
            break;

        case 'q':
            data.algorithm = FANN_TRAIN_QUICKPROP;
            printf("Doing Quickprop training with weight decay = %f and step change factor = %f\n", data.param1, data.param2);
            break;

        default:
            printf("Invalid training algorithm!\n");
            exit(1);
    }


    // Load binary training and verification data.
    data.train_data = TrainingDataLoad(data.train_file);
    data.verif_data = TrainingDataLoad(data.verif_file);

    // If the data could be read, continue.
    if ((data.train_data == NULL) || (data.verif_data == NULL)) {
        printf("Data load error!\n");
        fann_destroy_train(data.train_data);
        fann_destroy_train(data.verif_data);
        exit(1);
    } else {
        printf("Data loaded.\n");
    }

    // Try to load the specified input net
    data.ann = fann_create_from_file(data.net_in_file);

    // If load failed, create a new net. Let user know what happened.
    if (data.ann == NULL) {
        data.ann = fann_create_standard_array(NUM_LAYERS, layer_neurons);

        // Use a linear output layer, and symmetric sigmoid in hidden layer(s).
        fann_set_activation_function_output(data.ann, FANN_LINEAR);
        fann_set_activation_function_hidden(data.ann, FANN_SIGMOID_SYMMETRIC);

        fann_init_weights(data.ann, data.train_data);
        //fann_randomize_weights(data.ann, -2, 2);
        printf("Created new net.\n");
    }
    else {
        printf("Net loaded.\n");
    }


    // Set up the algorithm and parameters according to the arguments. Quit on error.
    switch (data.algorithm) {
        case FANN_TRAIN_INCREMENTAL:
            fann_set_training_algorithm(data.ann, data.algorithm);
            fann_set_learning_rate(data.ann, data.param1);
            fann_set_learning_momentum(data.ann, data.param2);
            break;

        case FANN_TRAIN_BATCH:
            fann_set_training_algorithm(data.ann, data.algorithm);
            fann_set_learning_rate(data.ann, data.param1);
            fann_set_learning_momentum(data.ann, data.param2);
            break;

        case FANN_TRAIN_RPROP:
            fann_set_training_algorithm(data.ann, data.algorithm);
            fann_set_rprop_increase_factor(data.ann, data.param1);
            fann_set_rprop_decrease_factor(data.ann, data.param2);
            fann_set_rprop_delta_min(data.ann, 0.0000001f); // 1e-7
            fann_set_rprop_delta_max(data.ann, 10.0f); // 1e1
            break;

        case FANN_TRAIN_QUICKPROP:
            fann_set_training_algorithm(data.ann, data.algorithm);
            fann_set_quickprop_decay(data.ann, data.param1);
            fann_set_quickprop_mu(data.ann, data.param2);
            break;

        // Shouldn't ever end up down here..
        default:
            fann_destroy(data.ann);
            fann_destroy_train(data.train_data);
            fann_destroy_train(data.verif_data);
            printf("Unknown training algorithm. (This error should not occur!)\n");
            exit(1);
    }


    // Save the net for now, and output some information.
    fann_save(data.ann, data.net_out_file);
    printf("Training for a maximum of %i epochs, with verification every %i epochs and target error %f\n", data.max_epochs, data.verif_epochs, data.target_error);

    // Set training callback, initialize best verification error and commence training.
    fann_set_callback(data.ann, &TrainingCallback);
    data.error_verif_best = fann_test_data(data.ann, data.verif_data);
    printf("\t  Verif. error: %f\n", NUM_OUTPUTS*data.error_verif_best);
    fann_train_on_data(data.ann, data.train_data, data.max_epochs, 1, data.target_error/NUM_OUTPUTS);

    // Will continue here when done training.
    printf("\nTraining ended.\n");

    // Free up memory before quitting.
    fann_destroy(data.ann);
    fann_destroy_train(data.train_data);
    fann_destroy_train(data.verif_data);

    return 0;
}
