/* Command line tool to convert datasets made in Matlab
 * (with misc/SaveBinaryData_GMM.m) to a format that
 * HTK can read.
 */

#include <sys/stat.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <gsl/gsl_matrix.h>

// Svein Gunnar Pettersen's library for reading/writing HTK feature files.
#include "ReadWriteFeat.h"

#define REVBYTEORDER 1 // Because HTK is weird.. Change this if errors occur!

// Load the specified training dataset made in Matlab, store as HTK feature file.
int main(int argc, char *argv[])
{
    FILE *inf_ptr;
    struct stat inf_stats;

    uint32_t num_vectors, num_dims;
    gsl_matrix *data_matrix;

    Header hdr;

    int revByteOrder = REVBYTEORDER;


    // Check the number of input arguments.
    if (argc != 3) {
        printf("Usage: %s <infile> <outfile>\n", argv[0]);
        exit(1);
    }

    // Get info for input file.
    if (stat(argv[1], &inf_stats)) {
        printf("Error: input %s does not exist!\n", argv[1]);
        exit(1);
    };

    // Open the input file. (Error checking might be redundant?)
    inf_ptr = fopen(argv[1], "rb");

    if (inf_ptr == NULL) {
        printf("Error: input %s could not be opened!\n", argv[1]);
        exit(1);
    }

    // Read the number of vectors and their dimension.
    if ((fread(&num_vectors, sizeof(uint32_t), 1, inf_ptr) < 1) || (fread(&num_dims, sizeof(uint32_t), 1, inf_ptr) < 1)) {
        printf("Error: could not read size of dataset!\n");
        exit(1);
    } else {
        printf("Dataset consists of %i vectors of dimension %i\n", num_vectors, num_dims);
    }

    if ( (inf_stats.st_size - 2*sizeof(uint32_t) - num_vectors*num_dims*sizeof(double) != 0) ) {
        printf("Error: size of %s differs from size reported in header!\n", argv[1]);
        exit(1);
    }


    // Allocate GSL matrix and read file into it.
    data_matrix = gsl_matrix_alloc(num_vectors, num_dims);
    if (data_matrix == NULL) {
        printf("Error: could not allocate memory for data!\n");
        exit(1);
    }
    if (gsl_matrix_fread(inf_ptr, data_matrix)) {
        printf("Error: GSL matrix fread failed!\n");
        exit(1);
    }

    // Set up the HTK header.
    hdr.parmKind = 9; // USER datatype.
    hdr.nSamples = num_vectors;
    hdr.sampSize = num_dims*sizeof(float);
    hdr.sampPeriod = 200000; // Does not really matter.. 200.000 * 100ns = 20ms.

    // Write output to file using Svein Gunnar's library.
    if (WriteFeatGsl(argv[2], hdr, revByteOrder, data_matrix)) {
        printf("Error: could not write output file %s!\n", argv[2]);
        exit(1);
    }

    fclose(inf_ptr);
    gsl_matrix_free(data_matrix);

    return 0;
}
