% Telephony Speech Bandwidth Extension via Neural networks.
clear

disp(' ');


% -- Set up some useful paths..
path(path, [pwd '/voicebox']);
path(path, [pwd '/fann_bind']);
path(path, [pwd '/misc']);


% Sound file to experiment on.
speaker = 'st17_se12';
filenum = '619';

sounddir = ['snd_data/test/' speaker '/'];
soundfile = [speaker '_' filenum '.raw'];


% Location of ANN file, with various configuration.
netdir = 'nets/';
netfile = '2L_12lpcc_ginfe_72_72_18cc.net'; f_highband = false; num_delta_offset = 0; f_liftering = true; k_lifter = 2;


F_sample = 16e3;  % Wideband sample rate.
frame_time = 20e-3; % Length of analysis frames in seconds.


% -- Settings for normed frame energy
% Forgetting factor
alpha = 0.96;
% Minimum energy search window length
N_min = 400;


% -- Create filters.
[b_pots, a_pots, pots_delay] = PotsFilter();
[b_aa, a_aa, aa_delay] = AntiAliasingFilter();
[b_hp, a_hp, hp_delay] = HighPassFilter();


% -- Setup of input to/output from neural net.
num_lpc_NB = 12;
num_cc_NB = num_lpc_NB;
num_dcc_NB = num_cc_NB;

if f_highband
    num_lpc_WB = 8;
    num_cc_WB = num_lpc_WB;
    num_dcc_WB = num_cc_WB;
else
    num_lpc_WB = 18;
    num_cc_WB = num_lpc_WB;
    num_dcc_WB = num_cc_WB;
end



% -- Load the neural net or GMM
ann = loadFann([netdir netfile]);



% -- Compute length of and distance between the analysis windows.
len_win_NB = round(F_sample/2 * frame_time /4)*4;
len_skip_NB = len_win_NB/2;

len_win_WB = 2*len_win_NB;
len_skip_WB = 2*len_skip_NB;



% -- Create analysis windows and cepstral lifters
win_NB = hamming(len_win_NB)';
win_WB = hamming(len_win_WB)';

if f_liftering
    L_NB = k_lifter*(num_cc_NB + 1); % +1 because n_cc_* is without c0.
    L_WB = k_lifter*(num_cc_WB + 1);
    
    lift_NB = 1 + L_NB/2 * sin((1:num_cc_NB) * pi / L_NB);
    lift_WB = 1 + L_WB/2 * sin((1:num_cc_WB) * pi / L_WB);
else
    lift_NB = ones(1, num_cc_NB);
    lift_WB = ones(1, num_cc_WB);
end


% -- Load the sound file.
t_x_WB = rawread([sounddir soundfile])';

% Make POTS version.
x_pots = filter(b_pots, a_pots, t_x_WB);
x_pots(1:pots_delay) = [];

% This is the input signal to the system
x_NB = downsample(x_pots, 2);
% Interpolate the input signal for pass-through.
x_WB = filter(b_aa, a_aa, 2*upsample(x_NB, 2));


% -- Perform LP-analysis of the narrowband signal.
x_frames_NB = enframe(x_NB, win_NB, len_skip_NB);
num_frames = size(x_frames_NB, 1);
lpc_NB = lpc(x_frames_NB', num_lpc_NB);

% Get the excitation.
e_NB = GetExcitation2(x_NB, lpc_NB, len_skip_NB);

% Compute feature parameters.
gi_NB = GradientIndex(x_frames_NB);
nfe_NB = NormedFrameEnergy(x_frames_NB, alpha, N_min, 0); % alpha = 0.96, N_min = 200
cc_NB = lpcar2cc(lpc_NB, num_cc_NB) .* repmat(lift_NB, num_frames, 1);

% -- Calculate target coefficients..
t_x_frames_WB = enframe(t_x_WB, win_WB, len_skip_WB);



% For high band.
if f_highband
    t_xhi_WB = filter(b_high, 1, t_x_WB);
    t_xhi_frames_WB = enframe(t_xhi_WB, win_WB, len_skip_WB);

    t_lpc_WB = lpc(t_xhi_frames_WB', num_lpc_WB);
    t_lpc_WB = t_lpc_WB(1:num_frames, :);
    t_cc_WB = lpcar2cc(t_lpc_WB, num_cc_WB);
else
    % For entire band.
    t_lpc_WB = lpc(t_x_frames_WB', num_lpc_WB);
    t_lpc_WB = t_lpc_WB(1:num_frames, :);
    t_cc_WB = lpcar2cc(t_lpc_WB, num_cc_WB);
end

    
if num_delta_offset > 0
%    e_frames_NB = enframe(e_NB, win_NB, len_skip_NB);
%    c0_NB = log(mean(e_frames_NB.^2, 2))/2;

%    dc_NB = ComputeDeltas([nfe_NB c0_NB cc_NB(:, 1:num_dcc_NB)], num_delta_offset);
    
%    input = [gi_NB((1+num_delta_offset):(end-num_delta_offset)) ...
%             nfe_NB((1+num_delta_offset):(end-num_delta_offset)) ...
%             cc_NB((1+num_delta_offset):(end-num_delta_offset), :) ...
%             dc_NB];
%     cc_WB = testFann(ann, input);
else
    input = [gi_NB nfe_NB cc_NB];
    
	cc_WB = testFann(ann, input);
end

% De-lifter.
cc_WB = cc_WB(:, 1:num_cc_WB) ./ repmat(lift_WB, num_frames-2*num_delta_offset, 1);

% -- Convert CCs to LPCs.
lpc_WB = CcToLpc(cc_WB, len_win_WB, num_lpc_WB);
newcc_WB = lpcar2cc(lpc_WB, num_cc_WB); % Does not matter much..


% -- Discard target LPCs and CCs that are gone due to delta offset.
t_lpc_WB = t_lpc_WB(1+num_delta_offset:end-num_delta_offset, :);
t_cc_WB = t_cc_WB(1+num_delta_offset:end-num_delta_offset, :);


cepsdist = sum((t_cc_WB - cc_WB).^2, 2);
disp(['Mean cepstral distance: ' num2str(mean(cepsdist))]);
disp(['       per coefficient: ' num2str(mean(cepsdist)/num_cc_WB)]);


% -- Calculate gain corrections.
% Frequency regions in which to compare filter gains.
if f_highband
    F_gain = [3.3e3 3.35e3];
else
    F_gain = 5e2:1e2:3.2e3;
end

gain = zeros(num_frames-2*num_delta_offset, 1);

for i_frame = 1:(num_frames-2*num_delta_offset)
        gain(i_frame) = ...
            mean(abs(freqz(1, lpc_NB(i_frame+num_delta_offset, :), F_gain, F_sample/2))) / ...
            mean(abs(freqz(1, lpc_WB(i_frame, :), F_gain, F_sample)));
end


% -- Extract the narrowband excitation, and extend it.
% Extend excitation for high and low band.
e_high_WB = ExtendExcitationHigh(e_NB);


y_WB =  x_WB + [ zeros(1, num_delta_offset*len_skip_WB) ...
                FilterExcitation2(e_high_WB(1+num_delta_offset*len_skip_WB:end), ...
                                  gain, lpc_WB, len_skip_WB) ];

LinuxPlay(x_pots, F_sample, 16, 'nn_narrowband.wav');
LinuxPlay(y_WB, F_sample, 16, 'nn_resynthesized.wav');
LinuxPlay(t_x_WB, F_sample, 16, 'nn_wideband.wav');
