% Telephony Speech Bandwidth Extension via Gaussian mixture modelling & MFCCs.
clear

disp(' ');

% -- Configure some useful paths..
path(path, [pwd '/voicebox']);
path(path, [pwd '/misc']);


% Sound file to experiment on.
sounddir = 'snd_data/test/st16_se04/';
soundfile = 'st16_se04_619.raw';


% -- Define some constants.
F_sample = 16e3;  % Wideband sample rate.
frame_time = 20e-3; % Length of analysis frames in seconds.

% These settings must fit with the GMM to be used!
gmmfile = 'gmms/mfcc_gain_5d_mix_64'; num_lpc_HB = 18; num_delta_offset = 5; f_lifter = false;


% -- Make filters.
[b_pots, a_pots, pots_delay] = PotsFilter();
[b_hp, a_hp, hp_delay] = HighPassFilter();
[b_bp, a_bp, bp_delay] = BandPassFilter();

a_preemph = [1 0.97];



% -- Numbers of filters and MFCCs to use in analysis.
num_filters_LB = 15;
num_filters_HB = 6;
num_filters = num_filters_LB + num_filters_HB;

num_mfcc_LB = 10; % Discard the 5 top MFCCs to conserve dimensionality.
num_dmfcc_LB = 6;

num_mfcc_HB = 6; % Use all 6 MFCCs.
num_dmfcc_HB = 4;


% -- Compute length of and distance between the analysis windows.
len_win = round(F_sample * frame_time /4)*4;
len_skip = len_win/2;



% -- Create DCTs, Mel filterbank and interpolating + frequency warping
% IDCT.
dct_LB = MakeDCT(num_filters_LB, num_filters_LB);
dct_HB = MakeDCT(num_filters_HB, num_filters_HB);
[mfbank, nl, nh] = melbankm(num_filters, len_win, F_sample, 0, 0.5, 't');

% Weighting to get approximately the same "area" for each filter.
% (This gives mean energy per window.)
for i_filter = 1:num_filters
    mfbank(i_filter, :) = mfbank(i_filter, :)/sum(mfbank(i_filter, :));
end

% Compute center Mel frequencies of the filters.
dM = frq2mel(F_sample/2)/(num_filters+1);

M_low_LB = dM * 1;
M_high_LB = dM * num_filters_LB;

M_low_HB = dM * (num_filters_LB + 1);
M_high_HB = dM * num_filters;

% Might be useful with the high/low band split frequency.
M_split = (M_high_LB + M_low_HB)/2;
F_split = mel2frq( M_split );

% Compute frequency vector for the FFT used to get autocorrelation
% function. Convert it to Mel scale.
len_fft = 2*len_win-1;
F_fft = (0:floor(len_fft/2))*F_sample/len_fft;
M_fft = frq2mel(F_fft);

% Find the indexes of frequencies to interpolate from low and high band
% MFCCs. Try to recreate the entire power spectrum because discontinuity
% results in strong poles near the edges..
i_interp_LB = (M_fft <= M_split);% & M_fft > M_low_LB);
i_interp_HB = (M_fft > M_split);

% Define what happens outside of the range of the Mel filter banks.
% (Continue with the last value, or interpolate further? The latter gives
% aliasing..)
M_fft((M_fft > M_high_LB) & ~i_interp_HB) = M_high_LB;
M_fft((M_fft < M_low_HB) & ~i_interp_LB)  = M_low_HB;
M_fft((M_fft < M_low_LB)) = M_low_LB;
M_fft((M_fft > M_high_HB)) = M_high_HB;

% Compute a frequency warped interpolation vector for the IDCT.
k_warp_LB = (M_fft - M_low_LB)/dM;
k_warp_HB = (M_fft - M_low_HB)/dM;


% Create interpolation IDCT.
iidct_LB = MakeIDCT(num_filters_LB, k_warp_LB(i_interp_LB));
iidct_HB = MakeIDCT(num_filters_HB, k_warp_HB(i_interp_HB));



% -- Create analysis window.
win = hamming(len_win)';

% -- Load the GMM to use for regression.
if num_delta_offset == 0
    GMM = LoadHtkGmm(gmmfile, num_mfcc_LB, num_mfcc_HB+1);
else
    GMM = LoadHtkGmm(gmmfile, num_mfcc_LB + num_dmfcc_LB, num_mfcc_HB + num_dmfcc_HB + 1);
end




% -- ANALYSIS STARTS HERE

% -- Load the sound file, make pre-emphasized version.
x_orig = rawread([sounddir soundfile])';
pex_orig = filter(1, a_preemph, x_orig);


% -- Make lowband version, and then a pre-emphasized version
x_NB = filter(b_pots, a_pots, x_orig);
x_NB(1:pots_delay) = []; % The LP filter causes a delay across the POTS band.
pex_NB = filter(1, a_preemph, x_NB);


% -- Analyze the original signal first.
pex_orig_frames = enframe(pex_orig, win, len_skip);
PEX_ORIG_frames = fft(pex_orig_frames, len_win, 2);

% Apply the Mel-filterbank to the squared DFT-magnitudes,
% then take the logarithm, (log Mel-filter energies)
logMFE_orig = log( abs( PEX_ORIG_frames(:, nl:nh) ).^2 * mfbank' );

% Apply the DCT to the low and high band log-energies
% separately.
%MFCC_LB_orig = logMFE_orig(:, (1:num_filters_LB)) * dct_LB';
MFCC_HB_orig = logMFE_orig(:, (1:num_filters_HB)+num_filters_LB) * dct_HB';


% -- Analyze the lowband signal.
pex_NB_frames = enframe(pex_NB, win, len_skip);
PEX_NB_frames = fft(pex_NB_frames, len_win, 2);

num_frames = size(pex_NB_frames, 1);

logMFE_NB = log( abs( PEX_NB_frames(:, nl:nh) ).^2 * mfbank' );

MFCC_LB = logMFE_NB(:, (1:num_filters_LB)) * dct_LB';
MFCC_LB = MFCC_LB .* repmat(lifter_LB, size(MFCC_LB, 1), 1);

% Get the highband MFCC and gain via GMM regression.
if num_delta_offset == 0
    temp = GmmRegress(GMM, MFCC_LB(:, 1:num_mfcc_LB));
else
    temp = GmmRegress(GMM, [MFCC_LB((1+num_delta_offset):(end-num_delta_offset), 1:num_mfcc_LB) ...
                            ComputeDeltas(MFCC_LB(:, 1:num_dmfcc_LB), num_delta_offset)]);
end

% Extract the MFCCs and gain. delta MFCCs are in between.
MFCC_HB = temp(:, 1:num_mfcc_HB) ./ repmat(lifter_HB, size(temp, 1), 1);
gain = k_gain * temp(:, end);

% -- Turn estimated MFCC_HB into all-pole filters.
RX = zeros(num_frames, 1+floor(len_fft/2)); % Only need half the DFT.
rx = zeros(num_frames, len_fft);
lpc_WB = zeros(num_frames-2*num_delta_offset, num_lpc_HB + 1);


for i_frame = 1+num_delta_offset:num_frames-num_delta_offset;
    % Recreate power spectrum.
%    RX(i_frame, i_interp_LB) = abs(PEX_NB_frames(i_frame, i_interp_LB)).^2; % This gives aliasing..
    RX(i_frame, i_interp_LB) = exp( iidct_LB*(MFCC_LB(i_frame, :) ./ lifter_LB)' );
    % Recreate high band from correct MFCCs.
%    RX(i_frame, i_interp_HB) = exp( (iidct_HB*[MFCC_HB_orig(i_frame, :) zeros(1, num_filters_HB-num_mfcc_HB)]') );
    % Recreate high band from estimated MFCCs.
    RX(i_frame, i_interp_HB) = exp( iidct_HB*[(MFCC_HB(i_frame-num_delta_offset, :)) zeros(1, num_filters_HB-num_mfcc_HB)]' );

    % Create autocorrelation function from power spectrum.
    if mod(len_fft,2) == 1
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end:-1:2)]);
    else
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end-1:-1:2)]);
    end

	% Estimate linear predictor coefficients via the autocorrelation method.
    lpc_WB(i_frame-num_delta_offset, :) = levinson(rx(i_frame, :), num_lpc_HB);
end

% Compute correct all-pole filters.
lpc_orig = lpc(pex_orig_frames', num_lpc_HB);


% -- Create excitation (BP-MGN)
bp_sig = filter(b_bp, a_bp, pex_NB);
e = abs(bp_sig).*randn(size(bp_sig));
e = filter(b_hp, a_hp, e);


% -- Synthesize high band.

% With correct filter.
%y = FilterExcitation2(e(1+num_delta_offset*len_skip:end-num_delta_offset*len_skip), gain, lpc_orig(1+num_delta_offset:num_frames-num_delta_offset,:), len_skip);
% With estimated filter.
y = FilterExcitation2(e(1+num_delta_offset*len_skip:end-num_delta_offset*len_skip), gain, lpc_WB, len_skip);


% Remove pre-emphasis.
y = filter(a_preemph, 1, y);

% Some simple deviation measure..
mean(sum((MFCC_HB_orig(1+num_delta_offset:num_frames-num_delta_offset, 1:num_mfcc_HB) - MFCC_HB).^2, 2))


LinuxPlay(x_NB, F_sample, 16, 'mfcc_narrowband.wav');
LinuxPlay(x_NB + [zeros(1, num_delta_offset*len_skip) y zeros(1, num_delta_offset*len_skip)], F_sample, 16, 'mfcc_resynthesized.wav');
LinuxPlay(x_orig, F_sample, 16, 'mfcc_wideband.wav');
%LinuxPlay(y, F_sample)
