% Create training data for GMMs in BWE System 2.

clear all
clc


% -- Set up paths to extra tools.
path(path, [pwd '/voicebox']);
%path(path, [pwd '/neuralnet']);
path(path, [pwd '/misc']);



% -- Set up location, names and number of soundfiles.
snd_dir = '/svog/NST_data/trainset/tale/';
% snd_subdirs = {'ace' 'aa3' 'ag6' 'a5i' 'adi' 'a8p' 'am5' 'afo' 'a5c' 'am1' 'aoj' 'a3x' 'aat' 'ace' 'abb' 'a1w' 'aah' 'akc' 'a84' 'abr'}; % 'aay' 'aou' 'ahq' 'a6t' 'ao7' 'a7b' 'ahg' 'ak9' 'ajp' 'aiv' 'ami' 'a2i' 'a7s' 'ahh' 'a6l' 'abx' 'ak3' 'a1x' 'a9x' 'ak0' 'alj' 'ahk' 'a00' 'a79' 'an7' 'a9u' 'abu' 'a76' 'akm' 'ao0' 'alh' 'a4a' 'acv' 'ahf' 'ajo' 'a7a' 'ac8' 'ac7' 'anp' 'agh' 'ald' 'a8s' 'acp' 'akv' 'a9j' 'amh' 'ao9' 'a7z' 'a73' 'ac0' 'ahw' 'amd' 'a6u' 'a9v' 'afs' 'aba' 'a25' 'a95' 'ahb' 'aev' 'a6w' 'abv' 'aip' 'a53' 'aj9' 'a5g' 'ani' 'abg' 'a4q' 'aif' 'ahm' 'a6i' 'aff' 'a0o' 'a5t' 'a6q' 'aj5' 'ah6' 'ago' 'a34'}; 100 speakers.
snd_subdirs = {'ae4';'aa3';'ag6';'a5i';'adi';'a8p';'am5';'afo';'a5c';'am1';'aoj';'a3x';'aat';'abb';'a1w';'aah';'akc';'a84';'abr';'aay';'aou';'ahq';'a6t';'ao7';'a7b';'ahg';'ak9';'ajp';'aiv';'ami';'a2i';'a7s';'ahh';'a6l';'abx';'ak3';'a1x';'a9x';'ak0';'alj';'ahk';'a00';'a79';'an7';'a9u';'abu';'a76';'akm';'ao0';'alh'}; % 50 speakers.
snd_nums = [(18:37) (43:57) (63:77) (83:102) (108:122) (128:142)]; % 100 of the longest sentences, all different between speakers.
snd_ext = '.bin';

num_subdirs = length(snd_subdirs);
num_files = length(snd_nums);

savedir = 'train_data/mfcc_pots2/';

% -- Set up parameters for the analysis.
F_sample = 16e3;
t_frame = 20e-3;

% How many previous and "future" frames to use for deltas. This applies to
% both high and low bands!
num_delta_offset = 5;


% Numbers of filters and MFCCs to use in analysis.
num_filters_LB = 15;
num_filters_HB = 6;
num_filters = num_filters_LB + num_filters_HB; % for 21 Mel-filters, 3.4 kHz lies almost right between filter 15 and 16.

% Numbers of MFCCs and delta MFCCs to train on.
% For estimation without memory.
if num_delta_offset == 0
    num_mfcc_LB = 10;
else
    % For estimation with memory.
    num_mfcc_LB = 10;
    num_dmfcc_LB = 6;
end

num_mfcc_HB = 6; % Use all 6 MFCCs.
num_dmfcc_HB = 4;

num_lpc_HB = 18; % How many predictor coefficients to recreate.



% -- Compute number of samples per frame, frame skip and overlap.

% This ensures 50 % overlap, and that the skip length is divisible
% by two (needed for the interpolation during filtering).
len_win = 4*round(F_sample *t_frame/4);
len_skip = len_win/2;


% -- Create DCTs and Mel filterbank.
dct_LB = MakeDCT(num_filters_LB, num_filters_LB);
dct_HB = MakeDCT(num_filters_HB, num_filters_HB);

[mfbank, nl, nh] = melbankm(num_filters, len_win, F_sample, 0, 0.5, 't');

% Normalize the filters.
for i_filter = 1:num_filters
    mfbank(i_filter, :) = mfbank(i_filter, :)/sum(mfbank(i_filter, :));
end

% Compute center Mel frequencies of the filters.
dM = frq2mel(F_sample/2)/(num_filters+1);

M_low_LB = dM * 1;
M_high_LB = dM * num_filters_LB;

M_low_HB = dM * (num_filters_LB + 1);
M_high_HB = dM * num_filters;

% Compute the frequency in the middle between the high and low band.
M_split = (M_high_LB + M_low_HB)/2;
F_split = mel2frq( M_split );

% Compute frequency vector for the FFT used to get autocorrelation
% function. Convert it to Mel scale.
len_fft = 2*len_win-1; % To avoid aliasing.
F_fft = (0:floor(len_fft/2))*F_sample/len_fft;
M_fft = frq2mel(F_fft);


% Find the indexes of frequencies to interpolate from low and high band
% MFCCs. Try to recreate the entire power spectrum because discontinuity
% results in strong resonant peaks near spectrum edges..
i_interp_LB = (M_fft <= M_split);
i_interp_HB = (M_fft > M_split);



% Specify a band to compare energies during gain-estimation.
i_compband = (F_fft >= mel2frq(M_low_HB)) & (F_fft <= mel2frq(M_high_HB));

% Define what happens outside of the range of the Mel filter banks.
M_fft((M_fft > M_high_LB) & ~i_interp_HB) = M_high_LB;
M_fft((M_fft < M_low_HB) & ~i_interp_LB)  = M_low_HB;
M_fft((M_fft < M_low_LB)) = M_low_LB;
M_fft((M_fft > M_high_HB)) = M_high_HB;

% Compute frequency warped interpolation vectors for the IDCTs.
k_warp_LB = (M_fft - M_low_LB)/dM;
k_warp_HB = (M_fft - M_low_HB)/dM;

% Create IDCT with interpolation.
iidct_LB = MakeIDCT(num_filters_LB, k_warp_LB(i_interp_LB)); % Simple way to interpolate the power spectrum.
iidct_HB = MakeIDCT(num_filters_HB, k_warp_HB(i_interp_HB));



% -- Make filters. (Should be the same is in the BWE system!)
[b_pots, a_pots, pots_delay] = PotsFilter();
[b_hp, a_hp, hp_delay] = HighPassFilter();
[b_bp, a_bp, bp_delay] = BandPassFilter();
a_preemph = [1 0.97];



% Window for use in analysis.
win = hamming(len_win)';

% Frequency vector for use in gain estimation.
F_win = (0:floor(len_win/2))*F_sample/len_win; 


% Allocate space for counting frames and storing data.
num_frames = zeros(num_subdirs*num_files, 1);
data_cells = cell(num_files*num_subdirs, 1);


% -- Make directory/file name for output data.

% Number of speakers.
subdir = [num2str(num_subdirs) 'sp_' ];

% Add descriptors for options to filename.
if num_delta_offset == 0
    subdir = [subdir 'noD_'];
else
    subdir = [subdir num2str(num_delta_offset) 'D_'];
end

subdir = [subdir num2str(num_mfcc_LB) 'LB_' num2str(num_mfcc_HB) 'HB_' num2str(num_lpc_HB) 'LPC/'];


% Save directly to binary to conserve a bunch of space.
disp(['Saving data to: ' savedir subdir]);
mkdir([savedir subdir]);


% Create a HTK script file.
fid_htk = fopen([savedir subdir 'htk_script'], 'w');


% -- Get MFCC for both narrow- and wideband signals from all files.
disp('Processing files...');
for i_subdir = 1:num_subdirs
%    filedir = [snd_dir snd_subdirs{i_subdir} '/'];
    filedir = [snd_dir '/'];
    
    % Load first file in subdirectory to estimate noise level.
    filename = [snd_subdirs{i_subdir} '_' num2str(1, '%04d') snd_ext];
    snd = binread([filedir filename]);

    snd_frames = enframe(snd, win, len_skip);
    frame_powers = mean(snd_frames.^2, 2);
    
    % Define a power limit to act as speech detection.
    power_lim = mean(frame_powers)*10^(6/10);
    
    
    % -- Start on the files to make training data from.
    for i_file = 1:num_files
        disp(' ');
        i_cell = i_file + (i_subdir - 1)*num_files;

        disp(['File ' num2str(i_cell) ' of ' num2str(num_subdirs*num_files)]);
        
        % Construct filename, then load the file.
        filename = [snd_subdirs{i_subdir} '_' num2str(snd_nums(i_file), '%04d')];
        snd = binread([filedir filename snd_ext]);

        len_snd = length(snd);
        
        % If the file exists/contained enough data, perform analysis.
        if (len_snd >= len_win)
            % Start by removing leading/trailing silence based on level.
            snd_frames = enframe(snd, win, len_skip);
            frame_powers = mean(snd_frames.^2, 2);
            
            % Find index of first and last frame above defined
            % speech detection level.
            i_start = find(frame_powers >= power_lim, 1, 'first');
            i_end = find(frame_powers >= power_lim, 1, 'last');

            % Discard the samples before and after these frames.
            i_speech = (1 + (i_start - 1)*len_skip):(len_win + (i_end - 1)*len_skip);
            snd = snd(i_speech);
            
            disp(['    removed ' num2str(100*(1 - length(i_speech)/len_snd)) ' %']);
            
            % Create POTS version.
            snd_NB = filter(b_pots, a_pots, snd);
            snd_NB(1:pots_delay) = [];
            
            % Apply preemphasis and enframe the signals.
            pesnd = filter(1, a_preemph, snd);
            pesnd_NB = filter(1, a_preemph, snd_NB);
            
            pesnd_frames = enframe(pesnd, win, len_skip);
            pesnd_NB_frames = enframe(pesnd_NB, win, len_skip);

            % Store the number of frames.
            num_frames(i_cell) = size(pesnd_NB_frames, 1);
            
            % Detect speech frames again, for later reduction of the analyzed
            % data.
            snd_frames = enframe(snd, win, len_skip);
            frame_powers = mean(snd_frames.^2, 2);
            i_speech = frame_powers >= power_lim;
            i_speech(num_frames(i_cell)+1:end) = [];
            
            % We cannot use frames that have no corresponding deltas MFCCs,
            % if it was specified..
            i_speech(1:num_delta_offset) = false;
            i_speech(end-num_delta_offset+1:end) = false;
            
            % Start analyzing.
            disp('  Making analysis data');

            % Get DFTs of num_frames(i_cell) number of frames.
            PESND_NB_frames = fft(pesnd_NB_frames, len_win, 2);
            PESND_frames = fft(pesnd_frames(1:num_frames(i_cell), :), len_win, 2);

            % Apply the Mel-filterbank to the squared DFT-magnitudes,
            % then take the logarithm to get log Mel-filter energies.
            logMFE_NB = log( abs( PESND_NB_frames(:, nl:nh) ).^2 * mfbank' );
            logMFE = log( abs( PESND_frames(:, nl:nh) ).^2 * mfbank' );
            
            % Apply the DCT to the low and high band log-energies
            % separately. Use narrowband signal for low band, and original
            % signal for high band.
            MFCC_LB = logMFE_NB(:, (1:num_filters_LB)) * dct_LB';
            MFCC_HB = logMFE(:, (1:num_filters_HB)+num_filters_LB) * dct_HB';

            % Allocate space for power spectrum, autocorrelation and
            % LP-filters.
            RX = zeros(num_frames(i_cell), 1+floor(len_fft/2));
            rx = zeros(num_frames(i_cell), len_fft);
            lpc_WB = zeros(num_frames(i_cell), num_lpc_HB + 1);
            

            % Estimate LPCs from reconstructed power spectrum.
            for i_frame = 1:num_frames(i_cell)
                % Recreate the power spectrum from the known, narrowband
                % MFCCs and the estimated high band MFCCs.
                %RX(i_frame, i_interp_LB) = abs( PESND_NB_frames(i_frame, i_interp_LB) ).^2; % This gives aliasing..
                temp1 = exp( iidct_LB*(MFCC_LB(i_frame, :) )' );
                temp2 = exp( (iidct_HB*[MFCC_HB(i_frame, 1:num_mfcc_HB)  zeros(1, num_filters_HB-num_mfcc_HB)]') );
                RX(i_frame, i_interp_LB) = temp1;
                RX(i_frame, i_interp_HB) = temp2;
                
                % The autocorrelation function has a symmetric spectrum.
                if mod(len_fft,2) == 1
                    rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end:-1:2)]);
                else
                    rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end-1:-1:2)]);
                end
                
                % Create LPCs using Levinson-Durbin.
                lpc_WB(i_frame, :) = levinson(rx(i_frame, :), num_lpc_HB);
            end


            % Recreate the high band, compare the energy with high band of
            % the original signal to estimate necessary gain.
            
            % Create excitation.
            bp_sig = filter(b_bp, a_bp, pesnd_NB);
            e = abs(bp_sig).*randn(size(bp_sig));

            % Synthesize using the estimated lpc_WBs and unity gain..
            syn_HB = FilterExcitation2(e, ones(num_frames(i_cell)), lpc_WB, len_skip);

            % Enframe and apply DFT.
            syn_HB_frames = enframe(syn_HB, win, len_skip);
            SYN_HB_frames = fft(syn_HB_frames, len_win, 2);

            % Compare energy in high band of the synthesized and original
            % signal to estimate what the gain should have been.
            gain = zeros(num_frames(i_cell), 1);

            for i_frame = 1:num_frames(i_cell)
                gain(i_frame) = sqrt(sum( abs(PESND_frames(i_frame, i_compband)).^2 )) / ...
                                sqrt(sum( abs(SYN_HB_frames(i_frame, i_compband)).^2 ));
            end

            
            % Compute delta coefficients if specified.
            if num_delta_offset > 0
                % Only if there are enough frames..
                if num_frames(i_cell) >= (1+2*num_delta_offset)
                    disp('    Making delta coefficients.');
                    
                    dMFCC_LB = ComputeDeltas(MFCC_LB(:, 1:num_dmfcc_LB), num_delta_offset);
                    dMFCC_HB = ComputeDeltas(MFCC_HB(:, 1:num_dmfcc_HB), num_delta_offset);
                else
                    disp('    Not enough frames for delta coefficients, ignoring file.');
                    num_frames(i_cell) = 0;
                end
            end

            % Adjust the frame count due to silence removal.
            num_frames(i_cell) = sum(i_speech);
            
            % If this file gives any training vectors
            if num_frames(i_cell) > 0
                % Only use training vectors that correspond to frames with
                % speech.
                if (num_delta_offset > 0)
                    data_in = [MFCC_LB(i_speech, 1:num_mfcc_LB) ...
                        dMFCC_LB(i_speech(1+num_delta_offset:end-num_delta_offset), :)];
                    
                    data_out = [MFCC_HB(i_speech, 1:num_mfcc_HB) ...
                        dMFCC_HB(i_speech(1+num_delta_offset:end-num_delta_offset), :) ...
                        gain(i_speech)];
                else
                    data_in = MFCC_LB(i_speech, 1:num_mfcc_LB);
                    
                    data_out = [MFCC_HB(i_speech, 1:num_mfcc_HB)...
                                gain(i_speech)];
                end
                
                % Save the data, convert to HTK format, then add to HTK script file.
                SaveBinaryData_GMM([savedir subdir filename '.bin'], data_in', data_out');
                system(['train_data/savetrain ' savedir subdir filename '.bin ' savedir subdir filename '.HTK']);
                fprintf(fid_htk, ['mfcc_pots/' subdir filename '.HTK\n']);

                % In case the data are needed afterwards..
%                data_cells{i_cell} = [data_in data_out];
            end
        end
    end
end


% Allocate space to concatenate all the data.
data_total = zeros(sum(num_frames), size(data_in,2) + size(data_out,2));

i_frames = 0;

% Populate data_total with data from the cells.
for i_cell = 1:length(data_cells)
    if num_frames(i_cell) > 0
        i_frames = i_frames(end) + (1:num_frames(i_cell));
        
%        data_total(i_frames, :) = data_cells{i_cell};
    end
end

% Close the HTK script file.
fclose(fid_htk);

disp(['Analyzed a total of ' num2str(sum(num_frames)) ' frames.']);
