function [y, lpc_est] = System2Synth(snd, gmm, num_delta_offset)
% Perform bandwidth extension using GMM-based regression.
%
% Input arguments
% snd : input POTS signal, 16 kHz sample rate
% gmm : GMM structure to use for regression of MFCCs
% num_delta_offset : number of frames for memory, if any
%
% Output
% y : bandwidth extended signal
% lpc_est : estimated LP-filters

% Assume no memory is used, if delta offset is not specified.
if ~exist('num_delta_offset', 'var')
    num_delta_offset = 0;
end

% Wideband sample rate.
F_sample = 16e3;
% Length of analysis frames in seconds.
frame_time = 20e-3; 

% Number of LPCs to recreate.
num_lpc = 18;

% Get necessary filters.
[b_hp, a_hp, hp_delay] = HighPassFilter();
[b_bp, a_bp, bp_delay] = BandPassFilter();
a_preemph = [1 0.97];

% Numbers of filters for low and high band in Mel-filter bank.
num_filters_LB = 15;
num_filters_HB = 6;
num_filters = num_filters_LB + num_filters_HB;

% Number of MFCCs and delta MFCCs for low band.
num_mfcc_LB = 10;
num_dmfcc_LB = 6;

% Use all 6 MFCCs.
num_mfcc_HB = 6; 
num_dmfcc_HB = 4;

% Length/distance of analysis windows.
len_win = round(F_sample * frame_time /4)*4;
len_skip = len_win/2;

% Create DCTs, Mel filterbank and interpolating + frequency warping
% IDCT.
dct_LB = MakeDCT(num_filters_LB, num_filters_LB);
dct_HB = MakeDCT(num_filters_HB, num_filters_HB);

[mfbank, nl, nh] = melbankm(num_filters, len_win, F_sample, 0, 0.5, 't');

% Normalize the Mel filter bank.
for i_filter = 1:num_filters
    mfbank(i_filter, :) = mfbank(i_filter, :)/sum(mfbank(i_filter, :));
end

% Compute center Mel frequencies of the upper and lower filters in the
% low and high band (LB and HB) filter sections.
dM = frq2mel(F_sample/2)/(num_filters+1);

M_low_LB = dM * 1;
M_high_LB = dM * num_filters_LB;

M_low_HB = dM * (num_filters_LB + 1);
M_high_HB = dM * num_filters;

% Compute the Mel frequency right between high and low band filter sections.
M_split = (M_high_LB + M_low_HB)/2;
F_split = mel2frq( M_split );

% Compute frequency vector for the FFT used to get autocorrelation
% function. Convert it to Mel scale.
len_fft = 2*len_win-1; % To avoid aliasing in autocorrelation function.
%len_fft = len_win; % This gives aliasing..
F_fft = (0:floor(len_fft/2))*F_sample/len_fft;
M_fft = frq2mel(F_fft);

% Find the indexes of frequencies to interpolate from low and high band
% MFCCs. (Recreate the entire power spectrum because discontinuity
% results in annoying resonances..)
i_interp_LB = (M_fft <= M_split);
i_interp_HB = (M_fft > M_split);

% Clamp the Mel frequencies to avoid aliasing outside of the frequency
% range from which the MFCCs stem.
M_fft((M_fft < M_low_LB)) = M_low_LB;
M_fft((M_fft > M_high_LB) & ~i_interp_HB) = M_high_LB;

M_fft((M_fft < M_low_HB) & ~i_interp_LB)  = M_low_HB;
M_fft((M_fft > M_high_HB)) = M_high_HB;

% Compute a frequency warped interpolation vector for the IDCT.
k_warp_LB = (M_fft - M_low_LB)/dM;
k_warp_HB = (M_fft - M_low_HB)/dM;

% Create interpolating IDCTs.
iidct_LB = MakeIDCT(num_filters_LB, k_warp_LB(i_interp_LB));
iidct_HB = MakeIDCT(num_filters_HB, k_warp_HB(i_interp_HB));

% Create analysis window..
win = hamming(len_win)';



% -- PROCESSING STARTS HERE!

% Apply pre-emphasis.
snd_pe = filter(1, a_preemph, snd);

% Enframe and compute DFT.
snd_pe_frames = enframe(snd_pe, win, len_skip);
SND_PE_frames = fft(snd_pe_frames, len_win, 2);

num_frames = size(snd_pe_frames, 1);

% Filter power spectrum with Mel filter bank, compute MFCCs.
logMFE = log( abs( SND_PE_frames(:, nl:nh) ).^2 * mfbank' );
MFCC_LB = logMFE(:, (1:num_filters_LB)) * dct_LB';

% Get the highband MFCC and gain via GMM regression, with or without memory.
if num_delta_offset == 0
    temp = GmmRegress(gmm, MFCC_LB(:, 1:num_mfcc_LB));
else
    temp = GmmRegress(gmm, [MFCC_LB((1+num_delta_offset):(end-num_delta_offset), 1:num_mfcc_LB) ...
                            ComputeDeltas(MFCC_LB(:, 1:num_dmfcc_LB), num_delta_offset)]);
end

% Get the high band MFCCs and gain. (delta MFCCs are in between)
MFCC_HB = temp(:, 1:num_mfcc_HB);
gain = temp(:, end);

% Allocate space for power spectrum, autocorrelation function and
% LP-filters.
RX = zeros(num_frames, 1+floor(len_fft/2)); % (only need half the DFT)
rx = zeros(num_frames, len_fft);
lpc_est = zeros(num_frames-2*num_delta_offset, num_lpc + 1);

% Recreate LP-filters per frame.
for i_frame = 1+num_delta_offset:num_frames-num_delta_offset;
    % Recreate power spectrum, both POTS and high band.
    % RX(i_frame, i_interp_LB) = abs(SND_PE_frames(i_frame, i_interp_LB)).^2; % This gives aliasing.
    temp1 = exp( iidct_LB*(MFCC_LB(i_frame, :) )' );
    temp2 = exp( (iidct_HB*[MFCC_HB(i_frame-num_delta_offset, :)  zeros(1, num_filters_HB-num_mfcc_HB)]') );

    RX(i_frame, i_interp_LB) = temp1;
    RX(i_frame, i_interp_HB) = temp2;%*temp1(end)/temp2(1); % Scale to avoid discontinuities? 

    % Create autocorrelation function from power spectrum.
    if mod(len_fft,2) == 1
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end:-1:2)]);
    else
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end-1:-1:2)]);
    end

	% Compute LPCs via the Levinson-Durbin algorithm.
    lpc_est(i_frame-num_delta_offset, :) = levinson(rx(i_frame, :), num_lpc);
end


% Create high band excitation (BP-MGN)
bp_env = abs(filter(b_bp, a_bp, snd_pe));
e_bpmgn = bp_env.*randn(size(bp_env));
e_bpmgn = filter(b_hp, a_hp, e_bpmgn);


% Filter excitation with the recreated LP-filters and estimated gains.
y = FilterExcitation2(e_bpmgn(1+num_delta_offset*len_skip:end-num_delta_offset*len_skip), gain, lpc_est, len_skip);

% Remove pre-emphasis, add to the original POTS signal.
% Pad with zeros for any missing frames due to delta computation.
y = snd + [zeros(1, num_delta_offset*len_skip) filter(a_preemph, 1, y) zeros(1, num_delta_offset*len_skip)];
