function [y_WB, lpc_WB] = System3Synth(snd, linmap, f_cconly)
% Perform bandwidth extension based on linear mapping of CCs (opt. +GI,NFE)
%
% Input arguments
% snd : input POTS signal, 8 kHz sample rate
% linmap : linear map to use
% f_cconly : specify if linear map is for CCs only (true/false)
%
% Output
% y_WB : bandwidth extended signal
% lpc_WB : estimated LP-filters

% Wideband sample rate.
F_sample = 16e3;
% Length of analysis frames in seconds.
frame_time = 20e-3;


% Forgetting factor for normed frame energy.
alpha = 0.96;
% Length of minimum frame energy search window.
N_minsearch = 400;


% Number of LPCs and CCs to/from linear map.
num_lpc_NB = 12;
num_cc_NB = num_lpc_NB;

num_lpc_WB = 18;
num_cc_WB = num_lpc_WB;

% Liftering multiplication factor.
k_lifter = 2;

% Length/spacing of analysis windows.
len_win_NB = round(F_sample/2 * frame_time /4)*4;
len_skip_NB = len_win_NB/2;

len_win_WB = 2*len_win_NB;
len_skip_WB = 2*len_skip_NB;


% Create window and lifters
win_NB = hamming(len_win_NB)';

L_NB = k_lifter*(num_cc_NB + 1); % +1 because n_cc_* is without c0.
L_WB = k_lifter*(num_cc_WB + 1);

lift_NB = 1 + L_NB/2 * sin((1:num_cc_NB) * pi / L_NB);
lift_WB = 1 + L_WB/2 * sin((1:num_cc_WB) * pi / L_WB);


% Enframe input signal, count number of frames.
snd_frames = enframe(snd, win_NB, len_skip_NB);
num_frames = size(snd_frames, 1);

% Compute LPCs and extract the excitation signal.
lpc_NB = lpc(snd_frames', num_lpc_NB);
e_NB = GetExcitation2(snd, lpc_NB, len_skip_NB);

% Generate input for linear map, perform estimation and de-lifter the CCs.
cc_NB = lpcar2cc(lpc_NB, num_cc_NB) .* repmat(lift_NB, num_frames, 1);

if f_cconly
    cc_WB = cc_NB*linmap;
else
    gi_NB = GradientIndex(snd_frames);
    nfe_NB = NormedFrameEnergy(snd_frames, alpha, N_minsearch, 0);
    
    cc_WB = [cc_NB gi_NB nfe_NB]*linmap;
end

cc_WB = cc_WB ./ repmat(lift_WB, num_frames, 1);

% Convert CCs to LPCs via autocorrelation function.
lpc_WB = CcToLpc(cc_WB, len_win_WB, num_lpc_WB);


% Calculate gain correction of estimated wideband filter.
F_gain = 5e2:50:3.2e3;

gain = zeros(num_frames, 1);

for i_frame = 1:num_frames
        gain(i_frame) = ...
            mean(abs(freqz(1, lpc_NB(i_frame, :), F_gain, F_sample/2))) / ...
            mean(abs(freqz(1, lpc_WB(i_frame, :), F_gain, F_sample)));
end


% Extend excitation for high band.
e_high_WB = ExtendExcitationHigh(e_NB);

[b_aa, a_aa, aa_delay] = AntiAliasingFilter();

y_WB =  filter(b_aa, a_aa, 2*upsample(snd, 2)) + FilterExcitation2(e_high_WB, gain, lpc_WB, len_skip_WB);
