function [e] = FilterExcitation2(x, gain, x_lpc, len_skip)
% Filter a signal in frames, using 2x interpolation of supplied
% gain factors and filter coefficients.
%
% Input arguments
% x : signal to filter
% gain : gain factors
% x_lpc : LP coefficients
% len_skip : number of samples between analysis windows (should be divisible by 4!), assume frame length is twice the skip length
%
% Output
% e : filtered signal

% Pre-allocate space for the output, compute number of frames.
e = zeros(size(x));
num_frames = size(x_lpc, 1);

% No interpolation for the first 2.5 subframes, as there is nothing to
% interpolate with.
i_samples = 1:(len_skip*5/4);
[e(i_samples), z_line] = filter(gain(1), x_lpc(1, :), x(i_samples));

% Filter the rest of the frames, perform interpolation for subframes
% inbetween analysis windows.
for i_frame = 2:num_frames-1
    % Interpolate gain and LPCs.
    gain_curr = 0.5*(gain(i_frame) + gain(i_frame - 1));
    lpc_curr = 0.5*(x_lpc(i_frame, :) + x_lpc(i_frame - 1, :));
    
    % Filter subframes.
    for i_subframe = 1:2
        i_samples = (1:len_skip/2) + len_skip*(i_frame - 1) + len_skip/2*(i_subframe - 1/2);
        [e(i_samples), z_line] = filter(gain_curr, lpc_curr, x(i_samples), z_line);
        % Second subframe is centered on an analysis window, do not interpolate.
        lpc_curr = x_lpc(i_frame, :);
        gain_curr = gain(i_frame);
    end
end


% The last frame also requires some special handling.
% Interpolation of first subframe.
gain_curr = 0.5*(gain(num_frames) + gain(num_frames - 1));
lpc_curr = 0.5*(x_lpc(num_frames, :) + x_lpc(num_frames - 1, :));

i_samples = (1:len_skip/2) + len_skip*(num_frames - 1) + len_skip/4;
[e(i_samples), z_line] = filter(gain_curr, lpc_curr, x(i_samples), z_line);

% No interpolation for the last 2.5 subframes.
i_samples = (1:(len_skip*5/4)) + len_skip*(num_frames - 1) + 3/4*len_skip;
e(i_samples) = filter(gain(num_frames), x_lpc(num_frames, :), x(i_samples), z_line);
