function [GMM] = LoadHtkGmm(filename, num_dimx, num_dimy)
% Loads the GMM from a single state HMM file created with HTK.
% Creates data structure for use in regression.
% (See GmmRegress.m)
%
% Input arguments
% filename : name of GMM file to load
% num_dimx : dimensions of x (what to estimate y from)
% num_dimy : dimensions of y (to estimate from x)
%
% Output
% GMM - the processed GMM, ready for use in regression


% Regex'es needed for file "navigation".
regex.head = '<VECSIZE>\s*(\d+)\s*<NULLD><\S+><(DIAGC|FULLC)>';
%regex.name = '~h "(\S+)"';
regex.nummix = '<NUMMIXES>\s*(\d+)';
regex.mix = '<MIXTURE>\s*(\d+)\s*([.\d\-\+eE]+)';
regex.mean = '<MEAN>\s*(\d+)';
regex.cov  = '<(VARIANCE|INVCOVAR)>\s*(\d+)';
%regex.gconst = '<GCONST>\s*([.\d\-\+eE]+)';

% Open file, give error if nonexistent.
htk_fid = fopen(filename, 'r');
if htk_fid < 0
    error('HTK GMM file does not exist!');
end

% Find <VECSIZE> to read dimensionality of GMM.
textline = fgetl(htk_fid);
found = regexp(textline, regex.head);
while (isempty(found) || ~found)
    textline = fgetl(htk_fid);
    found = regexp(textline, regex.head);
end

% If the dimensionality does not equal num_dimx + num_dimy,
% give a warning.
num_dims = sscanf(textline(1+length('<VECSIZE> '):end), '%d');
if (num_dimx + num_dimy) ~= num_dims
    disp('GMM dimension and summed dimensions of x and y do not match!')
end


% Find <NUMMIXES> to read number of mixtures, and output this number.
textline = fgetl(htk_fid);
found = regexp(textline, regex.nummix);
while (isempty(found) || ~found)
    textline = fgetl(htk_fid);
    found = regexp(textline, regex.nummix);
end
num_mixes = sscanf(textline(1+length('<NUMMIXES> '):end), '%d');
disp(['Number of mixtures: ' num2str(num_mixes)]);

% Preallocate space for GMM data structure.
GMM.apriori = zeros(num_mixes, 1);
GMM.mu = zeros(num_dims, num_mixes);
GMM.sigma = zeros(num_dims, num_dims, num_mixes);
GMM.mux = zeros(num_dimx, num_mixes);
GMM.muy = zeros(num_dimy, num_mixes);
GMM.invCxx = zeros(num_dimx, num_dimx, num_mixes);
GMM.constxx = zeros(num_mixes);
GMM.Cyx = zeros(num_dimy, num_dimx, num_mixes);


% Need this because HTK might drop mixtures, but does not update the
% mixture count when it does..
abort = false;


% Read the individual mixture components of the GMM, and create the
% data needed for regression.
% NB! Does not work for single-mixture GMMs??
for i_mix = 1:num_mixes
    % Find <MIXTURE> to locate next mixture..
    textline = fgetl(htk_fid);
    found = regexp(textline, regex.mix);
    while ((isempty(found) || ~found) && ~abort)
        textline = fgetl(htk_fid);

        % We need to abort if no line was read.
        if textline ~= -1
            found = regexp(textline, regex.mix);
        else
            abort = true;
        end
    end
    
    % Output a warning regarding number of mixtures if aborting.
    if abort
        disp(['Actual number of mixes lower than indicated in header! (' num2str(i_mix-1) ')'])
        num_mixes = i_mix-1;
        break;
    end

    % Get two floats. First is mixture number, second is apriori probability.
    temp = sscanf(textline(1+length('<MIXTURE> '):end), '%f', 2);
    GMM.apriori(i_mix) = temp(2);


    % Find <MEAN> to get mean vector of this mixture.
    textline = fgetl(htk_fid);
    found = regexp(textline, regex.mean);

    while (isempty(found) || ~found)
        textline = fgetl(htk_fid);
        found = regexp(textline, regex.mean);
    end

    % Get mean vector, split in x and y mean vectors.
    textline = fgetl(htk_fid);
    GMM.mu(:,i_mix) = sscanf(textline, '%f', num_dims);
    GMM.mux(:, i_mix) = GMM.mu(1:num_dimx, i_mix);
    GMM.muy(:, i_mix) = GMM.mu(num_dimx+(1:num_dimy), i_mix);

    % Find <INVCOVAR> to get inverted covariance matrix.
    textline = fgetl(htk_fid);
    found = regexp(textline, regex.cov);

    while (isempty(found) || ~found)
        textline = fgetl(htk_fid);
        found = regexp(textline, regex.cov);
    end

    temp = zeros(num_dims);

    % Read the inverted covariance matrix. Only upper triangle half is
    % in file, extrapolate the rest.
    for i_dim = 1:num_dims
        textline = fgetl(htk_fid);
        temp(i_dim, i_dim:end) = sscanf(textline, '%f');
        temp(i_dim+1:end, i_dim) = temp(i_dim, i_dim+1:end);
    end

    % Compute covariance matrix first, then the matrices and constant
    % needed for regression.
    GMM.sigma(:,:,i_mix) = inv(temp); 
    GMM.invCxx(:,:,i_mix) = inv(GMM.sigma(1:num_dimx, 1:num_dimx, i_mix));
    GMM.Cyx(:,:,i_mix) = GMM.sigma(num_dimx+(1:num_dimy), 1:num_dimx, i_mix);
    GMM.constxx(i_mix) = sqrt( abs( det( GMM.sigma(1:num_dimx, 1:num_dimx, i_mix) ) ) );
end


% Free remaining memory if stored number of mixes did not correspond with
% the header..
if abort
    GMM.apriori(num_mixes+1:end) = [];
    GMM.sigma(:,:,num_mixes+1:end) = [];
    GMM.mu(:,num_mixes+1:end) = [];
    GMM.mux(:,num_mixes+1:end) = [];
    GMM.muy(:, num_mixes+1:end) = [];
    GMM.invCxx(:,:, num_mixes+1:end) = [];
    GMM.Cyx(:,:, num_mixes+1:end) = [];
    GMM.constxx(num_mixes+1:end) = [];
end

% Close the GMM file.
fclose(htk_fid);
