function [lsd] = RMSLogSpectralDeviation(lpc_orig, lpc_est, gain_est, a_est)
% Compute RMS log spectral deviation in the band 4 - 7 kHz of reference and
% estimated LP-filters.
%
% Input arguments
% lpc_orig : reference LP-filters
% lpc_est : estimated LP-filter
% gain_est : correction gain for estimated LP-filters
% a_est : in case an all-zero filter is needed to correct response of lpc_est
%         (i.e. de-emphasis)
% 
% Output
% lsd : vector containing log spectral deviation for each frame.

% If a_est is not specified, default to 1. 
if ~exist('a_est', 'var')
    a_est = 1;
end

% Count number of frames, pre-allocate output variable
num_frames = size(lpc_est, 1);
lsd = zeros(num_frames, 1);

% Use the same band as Nour-Eldin & Kabal, with 50 Hz resolution.
F_lsd = 4e3:50:7e3;

% Compute RMS-LSD for each set of LP-filters.
for i_frame = 1:num_frames
    lsd(i_frame) = sqrt( mean( abs( ...
                       20*log10(abs( freqz(1, lpc_orig(i_frame, :), F_lsd, 16e3) )) ...
                     - 20*log10(gain_est(i_frame) * abs( freqz(a_est, lpc_est(i_frame, :), F_lsd, 16e3) )) ...
                   ).^2 ) );
end
