function [y, lpc_est] = sep_System2Synth_new(snd, gmm_mfcc, gmm_gain, num_delta_offset, num_mfcc)

% Assume no memory is used, if delta offset is not specified.
if ~exist('num_delta_offset', 'var')
    num_delta_offset = 0;
end

% -- Various analysis settings
F_sample = 16e3;  % Wideband sample rate.
frame_time = 20e-3; % Length of analysis frames in seconds.

% Number of LPCs to recreate.
num_lpc = 18;

% Get necessary filters.
[b_hp, a_hp, hp_delay] = HighPassFilter();
[b_bp, a_bp, bp_delay] = BandPassFilter();
a_preemph = [1 0.97];

% Numbers of filters and MFCCs to use.
num_filters_LB = 15;
num_filters_HB = 6;
num_filters = num_filters_LB + num_filters_HB;

num_mfcc_LB = num_mfcc(1);
num_dmfcc_LB = num_mfcc(2);

num_mfcc_HB = num_mfcc(3);
num_dmfcc_HB = num_mfcc(4);

% Create lifters. (Should not really matter in regression.)
lifter_LB = MakeLifter(10, num_filters_LB);
lifter_HB = MakeLifter(10, num_filters_HB);


% Length/distnace of analysis windows.
len_win = round(F_sample * frame_time /4)*4;
len_skip = len_win/2;


% Create DCTs, Mel filterbank and interpolating + frequency warping
% IDCT.
dct_LB = MakeDCT(num_filters_LB, num_filters_LB);
dct_HB = MakeDCT(num_filters_HB, num_filters_HB);

[mfbank, nl, nh] = melbankm(num_filters, len_win, F_sample, 0, 0.5, 't');

% Normalize the Mel filter bank.
for i_filter = 1:num_filters
    mfbank(i_filter, :) = mfbank(i_filter, :)/sum(mfbank(i_filter, :));
end

% Compute center Mel frequencies of the upper and lower filters in the
% low and high band (LB and HB) filter sections.
dM = frq2mel(F_sample/2)/(num_filters+1);

M_low_LB = dM * 1;
M_high_LB = dM * num_filters_LB;

M_low_HB = dM * (num_filters_LB + 1);
M_high_HB = dM * num_filters;

% Compute the Mel frequency right between high and low band filter sections.
M_split = (M_high_LB + M_low_HB)/2;
F_split = mel2frq( M_split );

% Compute frequency vector for the FFT used to get autocorrelation
% function. Convert it to Mel scale.
len_fft = 2*len_win-1; % To avoid aliasing in autocorrelation function.
%len_fft = len_win; % This gives aliasing..
F_fft = (0:floor(len_fft/2))*F_sample/len_fft;
M_fft = frq2mel(F_fft);

% Find the indexes of frequencies to interpolate from low and high band
% MFCCs. (Recreate the entire power spectrum because discontinuity
% results in annoying resonances..)
i_interp_LB = (M_fft <= M_split);
i_interp_HB = (M_fft > M_split);

% Clamp the Mel frequencies to avoid aliasing outside of the frequency
% range from which the MFCCs stem.
M_fft((M_fft < M_low_LB)) = M_low_LB;
M_fft((M_fft > M_high_LB) & ~i_interp_HB) = M_high_LB;

M_fft((M_fft < M_low_HB) & ~i_interp_LB)  = M_low_HB;
M_fft((M_fft > M_high_HB)) = M_high_HB;

% Compute a frequency warped interpolation vector for the IDCT.
k_warp_LB = (M_fft - M_low_LB)/dM;
k_warp_HB = (M_fft - M_low_HB)/dM;

% Create interpolation IDCT.
iidct_LB = MakeIDCT(num_filters_LB, k_warp_LB(i_interp_LB));
iidct_HB = MakeIDCT(num_filters_HB, k_warp_HB(i_interp_HB));


% Create analysis window..
win = hamming(len_win)';



% -- ANALYSIS STARTS HERE

% Apply pre-emphasis.
snd_pe = filter(1, a_preemph, snd);

% Enframe and compute DFT.
snd_pe_frames = enframe(snd_pe, win, len_skip);
SND_PE_frames = fft(snd_pe_frames, len_win, 2);

num_frames = size(snd_pe_frames, 1);

% Filter power spectrum with Mel filter bank, and compute MFCCs.
logMFE = log( abs( SND_PE_frames(:, nl:nh) ).^2 * mfbank' );
MFCC_LB = logMFE(:, (1:num_filters_LB)) * dct_LB';
MFCC_LB = MFCC_LB .* repmat(lifter_LB, size(MFCC_LB, 1), 1);

% Get the highband MFCC and gain via GMM regression, with or without memory.
input = [MFCC_LB((1+num_delta_offset):(end-num_delta_offset), 1:num_mfcc_LB) ...
                            ComputeDeltas(MFCC_LB(:, 1:num_dmfcc_LB), num_delta_offset)];
                        
MFCC_HB = GmmRegress(gmm_mfcc, input);
MFCC_HB(:, num_mfcc_HB+1:end) = [];
gain = GmmRegress(gmm_gain, input);

% Allocate space for power spectrum, autocorrelation function and
% LP-filters.
RX = zeros(num_frames, 1+floor(len_fft/2)); % (only need half the DFT)
rx = zeros(num_frames, len_fft);
lpc_est = zeros(num_frames-2*num_delta_offset, num_lpc + 1);


% Recreate LP-filters per frame.
for i_frame = 1+num_delta_offset:num_frames-num_delta_offset;
    % Recreate power spectrum, both POTS and high band.
    % RX(i_frame, i_interp_LB) = abs(SND_PE_frames(i_frame, i_interp_LB)).^2; % This gives aliasing.
    temp1 = exp( iidct_LB*(MFCC_LB(i_frame, :) ./ lifter_LB)' );
    temp2 = exp( (iidct_HB*[(MFCC_HB(i_frame-num_delta_offset, :) ./ lifter_HB(1:num_mfcc_HB)) zeros(1, num_filters_HB-num_mfcc_HB)]') );

    % Try to align the two power spectra..?
    RX(i_frame, i_interp_LB) = temp1;
    RX(i_frame, i_interp_HB) = temp2;

    % Create autocorrelation function from power spectrum.
    if mod(len_fft,2) == 1
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end:-1:2)]);
    else
        rx(i_frame, :) = ifft([0 RX(i_frame, 2:end) RX(i_frame, end-1:-1:2)]);
    end

	% Compute LPCs via the Levinson-Durbin algorithm.
    lpc_est(i_frame-num_delta_offset, :) = levinson(rx(i_frame, :), num_lpc);
end

% -- Synthesis starts here.

% Create excitation (BP-MGN)
bp_env = abs(filter(b_bp, a_bp, snd_pe));
e_bpmgn = bp_env.*randn(size(bp_env));
e_bpmgn = filter(b_hp, a_hp, e_bpmgn);

% -- Synthesize high band.

% Filter with the recreated LP-filters.
y = FilterExcitation2(e_bpmgn(1+num_delta_offset*len_skip:end-num_delta_offset*len_skip), gain, lpc_est, len_skip);

% Remove pre-emphasis, add to original POTS signal.
% Pad with zeros for any missing frames due to delta computation.
y = snd + [zeros(1, num_delta_offset*len_skip) filter(a_preemph, 1, y) zeros(1, num_delta_offset*len_skip)];
